import sys, os

sys.path.append(os.path.join(os.getcwd(), '..'))
from unittest import TestCase
from server.server import Server
from additionals.settings import DEFAULT_PORT


class TestServerTest(TestCase):

    def setUp(self):
        self.server = Server('', DEFAULT_PORT)
        self.bad_dict = {
            'response': 400,
            'ERROR': 'Bad Request'
        }
        self.good_dict = {
            'response': 200
        }

    def tearDown(self):
        self.server.sock.close()

    def test_check_presence_wo_time(self):
        self.assertEqual(self.server.check_presence({
            'action': 'presence',
            'user': {
                'account_name': 'Guest'
            }
        }), self.bad_dict)

    def test_check_presence_wo_action(self):
        self.assertEqual(self.server.check_presence({
            'time': 1.1,
            'user': {
                'account_name': 'Guest'
            }
        }), self.bad_dict)

    def test_check_presence_wo_account_name(self):
        self.assertEqual(self.server.check_presence({
            'action': 'presence',
            'time': 1.1}), self.bad_dict)

    def test_check_presence_wrong_action(self):
        self.assertEqual(self.server.check_presence({
            'action': 'hello_world',
            'time': 1.1,
            'user': {
                'account_name': 'Guest'
            }
        }), self.bad_dict)

    def test_check_presence_wrong_name(self):
        self.assertEqual(self.server.check_presence({
            'action': 'presence',
            'time': 1.1,
            'user': {
                'account_name': 'new_one'
            }
        }), self.bad_dict)

    def test_check_presence_ok(self):
        self.assertEqual(self.server.check_presence({
            'action': 'presence',
            'time': 1.1,
            'user': {
                'account_name': 'Guest'
            }
        }), self.good_dict)


if __name__ == '__main__':
    unittest.main()



import logging
import socket
from additionals.decos import Log
from .check_ip import check_first_octet, check_second_third_octets, check_forth_octet, check_ip


logger = logging.getLogger('app.server_')


@Log()
class PortVerifier:
    """
    check for ip port
    """
    def __set__(self, instance, value):
        if value < 1024 or value > 65535:
            logger.error(f'invalid port number {value}')
            raise ValueError(f'port is out off range 1024 - 65535, u typed {value}')
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name


@Log()
class HostVerifier:
    """
    check for ip address
    """
    def __set__(self, instance, value):
        if not check_ip(value):
            try:
                ip = socket.gethostbyname(value)
                sliced_ip = ip.split('.')
            except ValueError:
                logger.error(f'ip address is incorrect {value}')

        else:
            sliced_ip = value.split('.')

        if len(sliced_ip) == 4:
            for el in range(0, 4):
                if el == 0 and check_first_octet(sliced_ip[el]) \
                        or 0 < el < 3 and check_second_third_octets(sliced_ip[el]) \
                        or el == 3 and check_forth_octet(sliced_ip[el]):
                    pass
                else:
                    logger.error(f'ip is incorrect {value}')
                    raise ValueError('ip is incorrect')
        else:
            logger.error(f'ip is incorrect {value}')
            raise ValueError('ip is incorrect')

        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name






import unittest
import sys
import os
sys.path.append(os.path.join(os.getcwd(), '..'))
from unittest import TestCase
from additionals.utils import send_msg, receive_msg
from additionals.settings import MAX_CONNECTIONS, DEFAULT_IP, DEFAULT_PORT, ENCODING, PACK_CAPACITY
import json


class TestSocket:
    def __init__(self, test_data):
        self.test_data = test_data

    def send(self, data):
        self.got_to_send = data

    def recv(self, size):
        self.prepare_to_encode = json.dumps(self.test_data)
        return self.prepare_to_encode.encode(ENCODING)


class TestUtilsTest(TestCase):

    def setUp(self):

        self.test_send_good = {
            'action': 'presence',
            'time': 1.1,
            'user': {
                'account_name': 'Guest'
            }
        }
        self.test_send_bad = {
            'action': 'presence',
            'time': 1.1,
            'user': {
                'account_name': 'unknown'
            }
        }
        self.test_rcv_good = {
            'response': 200
        }
        self.test_rcv_bad = {
            'response': 400,
            'ERROR': 'Bad Request'
        }
        #good
        self.prepare_to_encode_msg_good = json.dumps(self.test_send_good)
        self.encoded_msg_good = self.prepare_to_encode_msg_good.encode(ENCODING)
        #bad
        self.prepare_to_encode_msg_bad = json.dumps(self.test_send_bad)
        self.encoded_msg_bad = self.prepare_to_encode_msg_bad.encode(ENCODING)
        #sockets
        self.test_sock_send = TestSocket(self.test_send_good)
        self.test_sock_rcv_good = TestSocket(self.test_rcv_good)
        self.test_sock_rcv_bad = TestSocket(self.test_rcv_bad)

    def tearDown(self):
        pass

    def test_send_msg(self):
        send_msg(self.test_sock_send, self.test_send_good)
        self.assertEqual(self.encoded_msg_good, self.test_sock_send.got_to_send)

    def test_rcv_msg_good(self):
        self.assertEqual(receive_msg(self.test_sock_rcv_good), self.test_rcv_good)


if __name__ == '__main__':
    unittest.main()


import logging

# connection settings

DEFAULT_PORT = 9350
DEFAULT_IP = '127.0.0.1'
PACK_CAPACITY = 4096     #max message length in bytes
MAX_CONNECTIONS = 5      #max client1 connections
ENCODING = 'utf-8'

# logging

LOGGING_LEVEL = logging.DEBUG



# JIM

ACTION = 'action'
PRESENCE = 'presence'
TIME = 'time'
ACCOUNT_NAME = 'account_name'
USER = 'user'
RESPONSE = 'response'
ERROR = 'ERROR'
MESSAGE = 'message'
SENDER = 'sender'
MESSAGE_TXT = 'mess_text'
TO = 'to'
FROM = 'from'
EXIT = 'exit'
RESPONSE_OK = '200 : OK'
GET_CONTACTS = 'get_contacts'
CONTACTS = 'contact_list'
ADD_CONTACT = 'add_contact'
REMOVE_CONTACT = 'remove_contact'
USERS_REQUEST = 'users_request'
PASSWORD = 'password'
REGISTER = 'register'
SECRET = 'is_that_joke_messenger'
PUB_KEY = 'public_key'
PUBLIC_KEY_REQUEST = 'public_key_request'
DATA = 'data'


# responses

RESPONSE_200 = {'response': 200}
RESPONSE_201 = {'response': 201,
                PUB_KEY: None}
RESPONSE_202 = {
    'response': 202,
    USERS_REQUEST: None
}
RESPONSE_203 = {
    'response': 203,
    GET_CONTACTS: None
}
RESPONSE_400 = {'response': 400, 'ERROR': 'Bad Request'}
RESPONSE_401 = {'response': 401, 'ERROR': 'Unauthorized'}
RESPONSE_403 = {'response': 403, 'ERROR': 'Forbidden'}
RESPONSE_412 = {'response': 412, 'ERROR': 'Bad Username'}
RESPONSE_511 = {
    RESPONSE: 511,
    DATA: None
}














import json
from .settings import ENCODING, PACK_CAPACITY
from .errors import IncorrectType, IncorrectDataToDecode
from additionals.decos import Log


@Log()
def send_msg(sock, message):
    """
    func to send message
    :param sock: connection socket
    :param message: dict
    :return:
    """
    msg_json = json.dumps(message)
    msg = msg_json.encode(ENCODING)
    sock.send(msg)


@Log()
def receive_msg(sender):
    """
    func to receive message
    :param sender: sender's socket
    :return: message dict
    """
    msg = sender.recv(PACK_CAPACITY)
    if isinstance(msg, bytes):
        msg_json = msg.decode(ENCODING)
        response = json.loads(msg_json)
        if isinstance(response, dict):
            return response
        raise IncorrectType
    raise IncorrectDataToDecode



import sys
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtWidgets import QMainWindow, QAction, qApp, QApplication, QDesktopWidget, QTableView, QPushButton, \
    QLineEdit, QLabel, QFileDialog, QDialog


def gui_create_model(database):
    """
    create model listing for connected users
    :param database: connected database
    :return: list of users
    """
    list_users = database.active_users_list()

    list = QStandardItemModel()
    list.setHorizontalHeaderLabels(['Username', 'IP', 'Port', 'Connection time'])
    for row in list_users:
        user, ip, port, time = row
        user = QStandardItem(user)
        user.setEditable(False)
        ip = QStandardItem(ip)
        ip.setEditable(False)
        port = QStandardItem(str(port))
        port.setEditable(False)
        time = QStandardItem(str(time.replace(microsecond=0)))
        time.setEditable(False)
        list.appendRow([user, ip, port, time])
    return list


def create_stat_model(database):
    """
    create model message statistics for connected users
    :param database: connected database
    :return: static list
    """
    hist_list = database.message_history()
    list = QStandardItemModel()
    list.setHorizontalHeaderLabels(
        ['Username', 'Last login', 'Sent messages', 'Received messages'])
    for row in hist_list:
        user, last_seen, sent, recvd = row
        user = QStandardItem(user)
        user.setEditable(False)
        last_seen = QStandardItem(str(last_seen.replace(microsecond=0)))
        last_seen.setEditable(False)
        sent = QStandardItem(str(sent))
        sent.setEditable(False)
        recvd = QStandardItem(str(recvd))
        recvd.setEditable(False)
        list.appendRow([user, last_seen, sent, recvd])
    return list


class MainGui(QMainWindow):
    """
    server main window GUI settings
    """
    def __init__(self):
        super().__init__()
        self.screen = QDesktopWidget().screenGeometry()
        self.width = int(self.screen.width() / 2)
        self.height = int(self.screen.height() / 1.8)
        self.x = int((self.screen.width() - self.width) / 2)
        self.y = int((self.screen.height() - self.height) / 2)
        self.initUI()

    def initUI(self):
        self.statusBar()
        self.exitAct = QAction('&Exit', self)
        self.exitAct.setShortcut('Ctrl+Q')
        self.exitAct.setStatusTip('Exit application')
        self.exitAct.triggered.connect(qApp.quit)

        self.refresh = QAction('&Refresh', self)
        self.refresh.setStatusTip('Refresh connected clients')

        self.history = QAction('&History', self)
        self.history.setStatusTip('Show clients history')

        self.server_config = QAction('&Server settings', self)
        self.server_config.setStatusTip('Edit server_ settings')

        self.menubar = self.menuBar()

        self.fileMenu = self.menubar.addMenu('&File')
        self.fileMenu.addAction(self.exitAct)

        self.settingsMenu = self.menubar.addMenu('&Settings')
        self.settingsMenu.addAction(self.server_config)

        self.ToolsMenu = self.menubar.addMenu('&Tools')
        self.ToolsMenu.addAction(self.refresh)
        self.ToolsMenu.addAction(self.history)

        self.setWindowTitle('Messenger')

        self.setFixedSize(self.width, self.height)
        self.move(self.x, self.y)
        self.active_users_table = QTableView(self)
        self.active_users_table.move(0, 45)
        self.active_users_table.setFixedSize(self.width, self.height - 65)

        self.show()


class HistoryWindow(QDialog):
    """
    server history window GUI settings
    """
    def __init__(self):
        super().__init__()
        self.initUI()

    def initUI(self):

        self.setWindowTitle('Statistics')
        self.setFixedSize(600, 700)
        self.setAttribute(Qt.WA_DeleteOnClose)

        self.close_button = QPushButton('Close', self)
        self.close_button.move(250, 650)
        self.close_button.clicked.connect(self.close)

        self.history_table = QTableView(self)
        self.history_table.move(10, 10)
        self.history_table.setFixedSize(580, 620)

        self.show()


class ConfigWindow(QDialog):
    """
    server settings window GUI settings
    """
    def __init__(self):
        super().__init__()
        self.initUI()

    def initUI(self):
        self.setFixedSize(365, 260)
        self.setWindowTitle('Server configuration')

        self.db_path_label = QLabel('Path to DB file: ', self)
        self.db_path_label.move(10, 10)
        self.db_path_label.setFixedSize(240, 15)

        self.db_path = QLineEdit(self)
        self.db_path.setFixedSize(250, 20)
        self.db_path.move(10, 30)
        self.db_path.setReadOnly(True)

        self.db_path_select = QPushButton('Open...', self)
        self.db_path_select.move(275, 28)

        def open_file_dialog():
            global dialog
            dialog = QFileDialog(self)
            path = dialog.getExistingDirectory()
            # path = path.replace('/', '\\')
            if path:
                self.db_path.clear()
                self.db_path.insert(path)

        self.db_path_select.clicked.connect(open_file_dialog)

        self.db_file_label = QLabel('BD filename: ', self)
        self.db_file_label.move(10, 68)
        self.db_file_label.setFixedSize(180, 15)

        self.db_file = QLineEdit(self)
        self.db_file.move(200, 66)
        self.db_file.setFixedSize(150 , 20)

        self.port_label = QLabel('Connection port:', self)
        self.port_label.move(10, 108)
        self.port_label.setFixedSize(180, 15)

        self.port = QLineEdit(self)
        self.port.move(200, 108)
        self.port.setFixedSize(150, 20)

        self.ip_label = QLabel('Connection ip:', self)
        self.ip_label.move(10, 148)
        self.ip_label.setFixedSize(180, 15)

        self.ip_label_note = QLabel('', self)
        self.ip_label_note.move(10, 168)
        self.ip_label_note.setFixedSize(500, 30)

        self.ip = QLineEdit(self)
        self.ip.move(200, 148)
        self.ip.setFixedSize(150, 20)

        self.save_btn = QPushButton('Save', self)
        self.save_btn.move(190, 220)

        self.close_button = QPushButton('Close', self)
        self.close_button.move(275, 220)
        self.close_button.clicked.connect(self.close)

        self.show()


def main():
    # app = QApplication(sys.argv)
    # ex = MainGui()
    # sys.exit(app.exec_())
    app = QApplication(sys.argv)
    ex = MainGui()
    ex.statusBar().showMessage('Test Statusbar Message')
    test_list = QStandardItemModel(ex)
    test_list.setHorizontalHeaderLabels(['Имя Клиента', 'IP Адрес', 'Порт', 'Время подключения'])
    test_list.appendRow([QStandardItem('1'), QStandardItem('2'), QStandardItem('3')])
    test_list.appendRow([QStandardItem('4'), QStandardItem('5'), QStandardItem('6')])
    ex.active_users_table.setModel(test_list)
    ex.active_users_table.resizeColumnsToContents()

    app.exec_()


if __name__ == '__main__':
    main()



import sys
import inspect
from logging import getLogger
from log.config import server_log_config, client_log_config

if 'server_' in sys.argv[0]:
    logger = getLogger('app.server_')
else:
    logger = getLogger('app.client1')


def log(func):
    """
    logger decorator func
    :param func:
    :return:
    """
    def save_to_log(self, *args, **kwargs):
        ordered_function = func(self, *args, **kwargs)

        logger.debug(f'the function "{func.__name__}" with params "{args}", "{kwargs}"'
                     f' has been ordered from module "{func.__module__}", called from "{inspect.stack()[1][3]}"'
                     f'', stacklevel=3)

        return ordered_function

    return save_to_log


class Log:
    """
    logger decorator class
    """
    def __init__(self):
        pass

    def __call__(self, func):

        def save_to_log(*args, **kwargs):
            ordered_function = func(*args, **kwargs)

            logger.debug(f'''(from class) the function "{''.join(inspect.stack()[1][4][0].split())}"''' 
                         f''' with params {args}, {kwargs}'''
                         f''' has been ordered from module {inspect.getmodule((inspect.stack()[1])[0])}, called from "{inspect.stack()[1][3]}"''',
                         stacklevel=2)

            return ordered_function

        return save_to_log




class IncorrectData(BaseException):

    def __str__(self):
        return 'received incorrect data'


class IncorrectType(BaseException):

    def __str__(self):
        return 'incorrect type'


class IncorrectDataToDecode(BaseException):

    def __str__(self):
        return 'incorrect data to decode. must be bytes'


class ClientError(BaseException):

    def __str__(self):
        return 'fuckup with starting client1'




import configparser
from datetime import datetime
from sqlalchemy import create_engine, MetaData, Table, Column, Integer, String, DateTime, ForeignKey, Text
from sqlalchemy.orm import mapper, sessionmaker
from sqlalchemy.sql import default_comparator


class ServerDB:
    """
    Main class for server database for SQLAlchemy
    """
    class AllUsers:
        """
        all users class
        """
        def __init__(self, username, pwd_hash):
            self.username = username
            self.id = None
            self.last_login = datetime.now()
            self.pwd_hash = pwd_hash

    class ActiveUsers:
        """
        active users class
        """
        def __init__(self, user_id, ip, port, login_time, keys):
            self.user = user_id
            self.ip = ip
            self.port = port
            self.login_time = login_time
            self.id = None
            self.keys = keys

    class LoginHistory:
        """
        login history class
        """
        def __init__(self, user_id, ip, port, date):
            self.user = user_id
            self.login_time = date
            self.ip = ip
            self.port = port
            self.id = None

    class Messages:
        """
        messages history class
        """
        def __init__(self, user_id, to_user, text, date):
            self.user = user_id
            self.to_user = to_user
            self.text = text
            self.date = date
            self.id = None

    class Contacts:
        """
        contacts relations class
        """
        def __init__(self, user_id, friend_id, date_add, date_remove=None, is_active=True):
            self.user = user_id
            self.friend = friend_id
            self.date_add = date_add
            self.date_remove = date_remove
            self.is_active = is_active
            self.id = None

    class MessagesCount:
        """
        messages count class
        """
        def __init__(self, user_id, updated=None):
            self.user = user_id
            self.sent = 0
            self.received = 0
            self.updated = updated

    def __init__(self, path):
        self.engine = create_engine(f'sqlite:///{path}', echo=False, pool_recycle=7200,
                                    connect_args={'check_same_thread': False})
        self.metadata = MetaData()
        all_users_table = Table('all_users', self.metadata,
                                Column('id', Integer, primary_key=True),
                                Column('username', String, unique=True),
                                Column('last_login', DateTime),
                                Column('pwd_hash', String))

        active_users_table = Table('active_users', self.metadata,
                                   Column('id', Integer, primary_key=True),
                                   Column('user', ForeignKey('all_users.id'), unique=True),
                                   Column('ip', String),
                                   Column('port', Integer),
                                   Column('login_time', DateTime),
                                   Column('keys', Text))

        login_history_table = Table('login_history', self.metadata,
                                    Column('id', Integer, primary_key=True),
                                    Column('user', ForeignKey('all_users.id')),
                                    Column('ip', String),
                                    Column('port', Integer),
                                    Column('login_time', DateTime))

        messages_table = Table('messages', self.metadata,
                               Column('id', Integer, primary_key=True),
                               Column('user', ForeignKey('all_users.id')),
                               Column('to_user', ForeignKey('all_users.id')),
                               Column('text', Text),
                               Column('date', DateTime))

        contacts_table = Table('contacts', self.metadata,
                               Column('id', Integer, primary_key=True),
                               Column('user', ForeignKey('all_users.id')),
                               Column('friend', ForeignKey('all_users.id')),
                               Column('date_add', DateTime),
                               Column('date_remove', DateTime, nullable=True, default=None),
                               Column('is_active', String, default=True))

        messages_count_table = Table('messages_count', self.metadata,
                                     Column('id', Integer, primary_key=True),
                                     Column('user', ForeignKey('all_users.id')),
                                     Column('sent', Integer, default=0),
                                     Column('received', Integer, default=0),
                                     Column('updated', DateTime, nullable=True))

        self.metadata.create_all(self.engine)
        mapper(self.AllUsers, all_users_table)
        mapper(self.ActiveUsers, active_users_table)
        mapper(self.LoginHistory, login_history_table)
        mapper(self.Messages, messages_table)
        mapper(self.Contacts, contacts_table)
        mapper(self.MessagesCount, messages_count_table)

        make_session = sessionmaker(bind=self.engine)
        self.session = make_session()

        self.session.query(self.ActiveUsers).delete()
        self.session.commit()

    def user_login(self, username, ip, port, pwd_hash, keys):
        """
        user login func
        :param username: username
        :param ip: ip address
        :param port: ip port
        :param pwd_hash: user password hash
        :param keys: user public key
        :return:
        """
        check_in_db = self.session.query(self.AllUsers).filter_by(username=username)

        if check_in_db.count():
            user = check_in_db.first()
            user.last_login = datetime.now()
        else:
            user = self.AllUsers(username, pwd_hash)
            self.session.add(user)
            self.session.commit()

        new_active_user = self.ActiveUsers(user.id, ip, port, datetime.now(), keys)
        self.session.add(new_active_user)

        new_history = self.LoginHistory(user.id, ip, port, datetime.now())
        self.session.add(new_history)

        if not self.session.query(self.MessagesCount).filter_by(user=user.id).count():
            new_messages_count = self.MessagesCount(user.id)
            self.session.add(new_messages_count)

        self.session.commit()

    def user_logout(self, username):
        """
        user logout func
        :param username: username
        :return:
        """
        user = self.session.query(self.AllUsers).filter_by(username=username).first()
        self.session.query(self.ActiveUsers).filter_by(user=user.id).delete()
        self.session.commit()

    def get_message(self, username, to_user, text):
        user = self.session.query(self.AllUsers).filter_by(username=username).first()
        other_user = self.session.query(self.AllUsers).filter_by(username=to_user).first()
        new_message = self.Messages(user.id, other_user.id, text, datetime.now())
        self.session.add(new_message)
        self.session.commit()

    def users_list(self):
        """
        list of all users
        :return: users queryset
        """
        query = self.session.query(
            self.AllUsers.username,
            self.AllUsers.last_login
        )
        return query.all()

    def active_users_list(self):
        """
        list of active users
        :return: active users queryset
        """
        query = self.session.query(
            self.AllUsers.username,
            self.ActiveUsers.ip,
            self.ActiveUsers.port,
            self.ActiveUsers.login_time
        ).join(self.AllUsers)
        return query.all()

    def login_history(self, username=None):
        """
        login user history
        :param username: username
        :return: login history queryset
        """
        query = self.session.query(
            self.AllUsers.username,
            self.LoginHistory.ip,
            self.LoginHistory.port,
            self.LoginHistory.login_time
        ).join(self.AllUsers)

        if username:
            query = query.filter(self.AllUsers.username == username)

        return query.all()

    def messages_list(self, username=None):
        """
        get user message history
        :param username: username
        :return: messages history queryset
        """
        query = self.session.query(
            self.AllUsers.username,
            self.Messages.to_user,
            self.Messages.text,
            self.Messages.date
        ).join(self.AllUsers)

        if username:
            query = query.filter(self.AllUsers.username == username)

        return query.all()

    def add_contact(self, user, friend):
        """
        func to add new contact and ave in database
        :param user: username
        :param friend: contact name
        :return:
        """
        user = self.session.query(self.AllUsers).filter_by(username=user).first()
        friend = self.session.query(self.AllUsers).filter_by(username=friend).first()

        if not friend or self.session.query(self.Contacts).filter_by(
                user=user.id, friend=friend.id, is_active=True).count():
            return
        elif self.session.query(self.Contacts).filter_by(user=user.id, friend=friend.id, is_active=False).count():
            contact = self.session.query(self.Contacts).filter_by(
                user=user.id, friend=friend.id, is_active=False).first()
            contact.is_active = True
        else:
            contact = self.Contacts(user.id, friend.id, datetime.now())
        self.session.add(contact)
        self.session.commit()

    def delete_contact(self, user, friend):
        """
        func to deactivate (delete) contact
        :param user: username
        :param friend: contact name
        :return:
        """
        user = self.session.query(self.AllUsers).filter_by(username=user).first()
        friend = self.session.query(self.AllUsers).filter_by(username=friend).first()

        if not friend or not self.session.query(self.Contacts).filter_by(user=user.id, friend=friend.id).count():
            return

        ex_friend = self.session.query(self.Contacts).filter_by(user=user.id, friend=friend.id).first()
        ex_friend.is_active = False
        ex_friend.date_remove = datetime.now()
        self.session.commit()

    def get_contact(self, username):
        """
        func returns users contacts list
        :param username: username
        :return: contacts lsit
        """
        user = self.session.query(self.AllUsers).filter_by(username=username).one()

        query = self.session.query(self.Contacts, self.AllUsers.username).filter_by(user=user.id). \
            join(self.AllUsers, self.Contacts.friend == self.AllUsers.id)

        query = query.filter(self.Contacts.is_active == True)

        return [contact[1] for contact in query.all()]

    def messages_count(self, sender, receiver):
        """
        func counts messages for each user
        :param sender: username
        :param receiver: username
        :return:
        """
        sender = self.session.query(self.AllUsers).filter_by(username=sender).first().id
        receiver = self.session.query(self.AllUsers).filter_by(username=receiver).first().id

        sender_id = self.session.query(self.MessagesCount).filter_by(user=sender).first()
        sender_id.sent += 1
        sender_id.updated = datetime.now()

        receiver_id = self.session.query(self.MessagesCount).filter_by(user=receiver).first()
        receiver_id.received += 1
        receiver_id.updated = datetime.now()

        self.session.commit()

    def get_messages_count(self, username=None):
        """
        returns count of messages
        :param username: username
        :return: queryset
        """
        query = self.session.query(
            self.AllUsers.username,
            self.MessagesCount.sent,
            self.MessagesCount.received).join(self.AllUsers)

        if username:
            query = query.filter(self.AllUsers.username == username)

        return query.all()

    def message_history(self):
        """
        returns messages for user
        :return: messages history queryset
        """
        query = self.session.query(
            self.AllUsers.username,
            self.AllUsers.last_login,
            self.MessagesCount.sent,
            self.MessagesCount.received
        ).join(self.AllUsers)

        return query.all()

    def check_user(self, user):
        """
        func user existing
        :param user: username
        :return: bool
        """
        if self.session.query(self.AllUsers).filter_by(username=user).count():
            return True
        else:
            return False

    def check_pwd(self, user):
        """
        returns password hash for user
        :param user: username
        :return: password hash, str
        """
        query = self.session.query(
            self.AllUsers.pwd_hash
        ).filter_by(username=user).first()
        return query

    def get_keys(self, name):
        """
        returns public key of user
        :param name:
        :return:
        """
        user = self.session.query(self.AllUsers).filter_by(username=name).one()
        user = self.session.query(self.ActiveUsers).filter_by(user=user.id).first()
        return user.keys


if __name__ == '__main__':
    server_config = configparser.ConfigParser()
    server_config.read('server_.ini')
    db = ServerDB(server_config['SETTINGS']['db_file_name'])
    db.user_login('test_1', '1.1.1.1', 0000, 1234, 123)
    db.user_login('gumdrop', '2.2.2.2', 0000, 1234, 123)
    # db.user_login('test_3', '2.2.2.2', 0000)

    # print(db.users_list())
    print(db.active_users_list())

    db.user_logout('test_1')

    print(db.active_users_list())
    # print(db.check_pwd('test_1')[0])
    print(db.get_keys('gumdrop'))
    # print(db.login_history('test_1'))
    # print(db.users_list())
    # # db.get_message('test_1', 'test_2', 'hi,man!')
    # # print(db.messages_list())
    # db.add_contact('test_1', 'test_2')
    # db.add_contact('test_1', 'test_3')
    # # db.delete_contact('test_1', 'test_2')
    # print(db.get_contact('test_1'))
    # db.messages_count('test_1', 'test_2')
    # db.messages_count('test_2', 'test_1')
    # print(db.get_messages_count())



import dis


class ServerVerifier(type):
    """
    meta class to check child class methods
    """
    def __init__(self, name, bases, dict):

        methods_lst = []
        func_lst = []
        for el in dict:
            try:
                instruction_bytes = dis.get_instructions(dict[el])
            except TypeError:
                pass
            else:
                for value in instruction_bytes:
                    if value.opname == 'LOAD_METHOD':
                        if value.argval not in methods_lst:
                            methods_lst.append(value.argval)

                    if value.opname == 'LOAD_GLOBAL':
                        if value.argval not in func_lst:
                            func_lst.append(value.argval)

        if 'connect' in methods_lst:
            raise TypeError('error method in client1 class')

        if not ('AF_INET' in methods_lst and 'SOCK_STREAM' in methods_lst):
            pass
        else:
            raise TypeError('incorrect initialisation of socket')

        super().__init__(name, bases, dict)


if __name__ == '__main__':
    pass



import argparse
import configparser
import os
import select
import sys
import threading
import time
from collections import deque
from logging import getLogger
from socket import socket, AF_INET, SOCK_STREAM
from PyQt5.QtCore import QTimer
from PyQt5.QtWidgets import QApplication, QMessageBox
from additionals.server_meta import ServerVerifier
from additionals.server_descriptor import PortVerifier, HostVerifier
from additionals.settings import RESPONSE_200, RESPONSE_400, ACTION, ACCOUNT_NAME, TIME, PRESENCE, USER, MESSAGE, \
    MESSAGE_TXT, SENDER, ERROR, TO, \
    EXIT, ADD_CONTACT, REMOVE_CONTACT, RESPONSE_202, GET_CONTACTS, USERS_REQUEST, RESPONSE_203, PASSWORD, REGISTER,\
    RESPONSE_412, PUB_KEY, PUBLIC_KEY_REQUEST, RESPONSE_511, DATA, RESPONSE_401, RESPONSE_403
from additionals.utils import send_msg, receive_msg
from additionals.decos import Log
from server.server_database import ServerDB
from server.server_gui import MainGui, gui_create_model, create_stat_model, ConfigWindow, HistoryWindow

new_connection = False
conflag_lock = threading.Lock()


@Log()
def arg_parser(default_address, default_port):
    """
    argument parser for server_.py
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('-a', default=default_address, nargs='?', help='ip address')
    parser.add_argument('-p', default=default_port, type=int, nargs='?', help=':port')
    namespace = parser.parse_args(sys.argv[1:])
    addr = namespace.a
    port = namespace.p
    return addr, port


class Server(threading.Thread, metaclass=ServerVerifier):
    """
    Server Main Class
    """
    port = PortVerifier()
    ip = HostVerifier()

    def __init__(self, ip_addr, ip_port, database):
        """
        server_ initialization
        @param ip_addr: ip address
        @param ip_port: ip port
        @param database: connected database
        """
        self.database = database
        self.logger = getLogger('app.server_')
        self.ip = ip_addr
        self.port = ip_port
        self.connected_clients = deque()
        self.messages_lst = []
        self.r_clients = deque()
        self.w_clients = deque()
        self.e_clients = deque()
        self.names = {}
        self.authentication_lst = {}
        super().__init__()

    @Log()
    def init_socket(self):
        """
        create socket connection
        """
        self.logger.warning(f'server_ starts with {self.ip}:{self.port}')
        self.sock = socket(AF_INET, SOCK_STREAM)
        self.logger.debug(f'server_ successfully started socket {self.sock}')
        self.sock.bind((self.ip, self.port))
        self.sock.settimeout(0.5)
        self.logger.debug('server_ successfully got socket')
        self.sock.listen()
        self.logger.debug('server_ started to listen socket')

    def run(self):
        global new_connection
        self.init_socket()

        while True:
            try:
                self.client, self.addr = self.sock.accept()
                self.authentication_lst[self.client] = round(time.time())
                self.logger.debug(f'clients connected {self.client}')
            except OSError:
                pass
            else:
                self.connected_clients.append(self.client)
            try:
                if self.connected_clients:
                    self.r_clients, self.w_clients, self.e_clients = select.select(self.connected_clients,
                                                                                   self.connected_clients,
                                                                                   [], 0)
            except OSError:
                pass
            if self.r_clients:
                for client_to_receive in self.r_clients:
                    try:
                        self.message_handler(receive_msg(client_to_receive), client_to_receive)
                    except Exception as e:
                        print(e)
                        self.get_name(client_to_receive)
                        with conflag_lock:
                            new_connection = True

            for message in self.messages_lst:
                try:
                    self.process_message(message, self.w_clients)
                except (ConnectionAbortedError, ConnectionError, ConnectionResetError, ConnectionRefusedError):
                    self.logger.info(f'Connection with {message[TO]} has been lost')
                    self.connected_clients.remove(self.names[message[TO]])
                    self.database.user_logout(message[TO])
                    del self.names[message[TO]]
                    with conflag_lock:
                        new_connection = True
            self.messages_lst.clear()

    @Log()
    def authenticate_request(self, name, pwd):
        """
        check entered users password
        @param name: username
        @param pwd: password
        @return: bool
        """
        query_pwd = self.database.check_pwd(name)
        if pwd == query_pwd[0]:
            return True
        return False

    @Log()
    def process_message(self, message, listen_socks):
        """
        send messages func
        @param message: dict
        @param listen_socks: connected sockets
        @return:
        """
        if message[TO] in self.names and self.names[message[TO]] in listen_socks:
            send_msg(self.names[message[TO]], message)
            self.logger.info(f'Message to {message[TO]} has been sent from {message[SENDER]}.')
        elif message[TO] in self.names and self.names[message[TO]] not in listen_socks:
            raise ConnectionError
        else:
            self.logger.error(
                f'User {message[TO]} is not registered on server_, send message failed.')

    @Log()
    def message_handler(self, message, client):
        """
        incoming messages handler
        @param message: dict
        @param client: username
        @return:
        """
        global new_connection
        print(message)
        if ACTION in message and message[ACTION] == REGISTER and TIME in message \
                and USER in message and client in self.authentication_lst:
            if not message[USER][ACCOUNT_NAME] in self.names.keys()\
                    and not self.database.check_user(message[USER][ACCOUNT_NAME]):
                self.names[message[USER][ACCOUNT_NAME]] = client
                del self.authentication_lst[client]
                client_ip, client_port = client.getpeername()
                self.database.user_login(message[USER][ACCOUNT_NAME], client_ip, client_port,
                                         message[USER][PASSWORD], message[USER][PUB_KEY])
                self.send(client, RESPONSE_200)
                with conflag_lock:
                    new_connection = True
                return
            else:
                response = RESPONSE_412
                response[ERROR] = f'client_ with name "{message[USER][ACCOUNT_NAME]}" is registered already'
                self.send(client, response)
                self.connected_clients.remove(client)
                del self.authentication_lst[client]
                client.close()
                return

        elif ACTION in message and message[ACTION] == PRESENCE and TIME in message \
                and USER in message and client in self.authentication_lst and \
                self.authenticate_request(message[USER][ACCOUNT_NAME], message[USER][PASSWORD]) and \
                self.database.check_user(message[USER][ACCOUNT_NAME]):
            if not message[USER][ACCOUNT_NAME] in self.names.keys():
                self.names[message[USER][ACCOUNT_NAME]] = client
                del self.authentication_lst[client]
                client_ip, client_port = client.getpeername()
                self.database.user_login(message[USER][ACCOUNT_NAME], client_ip, client_port,
                                         message[USER][PASSWORD], message[USER][PUB_KEY])
                self.send(client, RESPONSE_200)
                with conflag_lock:
                    new_connection = True
                return
            else:
                response = RESPONSE_401
                response[ERROR] = f'client_ with name "{message[USER][ACCOUNT_NAME]}" is in chat already'
                self.send(client, response)
                self.connected_clients.remove(client)
                del self.authentication_lst[client]
                client.close()
                return

        elif ACTION in message and message[ACTION] == PRESENCE and TIME in message \
                and USER in message and client in self.authentication_lst and not\
                self.database.check_user(message[USER][ACCOUNT_NAME]):
            response = RESPONSE_403
            response[ERROR] = f'client_ with name "{message[USER][ACCOUNT_NAME]}" is not registered'
            self.send(client, response)
            self.connected_clients.remove(client)
            del self.authentication_lst[client]
            client.close()
            return

        elif ACTION in message and message[ACTION] == PRESENCE and TIME in message \
                and USER in message and client in self.authentication_lst and \
                self.database.check_user(message[USER][ACCOUNT_NAME]) and not \
                self.authenticate_request(message[USER][ACCOUNT_NAME], message[USER][PASSWORD]):
            response = RESPONSE_401
            response[ERROR] = f'wrong password'
            self.send(client, response)
            self.connected_clients.remove(client)
            del self.authentication_lst[client]
            client.close()
            return

        elif ACTION in message and message[ACTION] == MESSAGE and TIME in message \
                and MESSAGE_TXT in message and SENDER in message and TO in message \
                and message[SENDER] not in self.authentication_lst:
            if message[TO] in self.names:
                self.messages_lst.append(message)
                self.database.messages_count(message[SENDER], message[TO])
                self.send(client, RESPONSE_200)
            else:
                response = RESPONSE_400
                response[ERROR] = 'client_ is not authorized'
                self.send(client, response)
            return

        elif ACTION in message and message[ACTION] == EXIT and TIME in message \
                and SENDER in message and self.names[message[SENDER]] == client \
                and message[SENDER] not in self.authentication_lst:
            self.database.user_logout(message[SENDER])
            self.connected_clients.remove(client)
            self.names.pop(message[SENDER])
            self.r_clients.remove(client)
            self.w_clients.remove(client)
            client.close()
            with conflag_lock:
                new_connection = True
            return

        elif ACTION in message and message[ACTION] == GET_CONTACTS and SENDER in message and \
                self.names[message[SENDER]] == client and message[SENDER] not in self.authentication_lst:
            self.response = RESPONSE_203
            self.response[GET_CONTACTS] = self.database.get_contact(message[SENDER])
            self.send(client, self.response)
            return

        elif ACTION in message and message[ACTION] == ADD_CONTACT and ACCOUNT_NAME in message and SENDER in message \
                and self.names[message[SENDER]] == client and message[SENDER] not in self.authentication_lst:
            self.database.add_contact(message[SENDER], message[ACCOUNT_NAME])
            self.send(client, RESPONSE_200)
            return

        elif ACTION in message and message[ACTION] == REMOVE_CONTACT and ACCOUNT_NAME in message and SENDER in message \
                and self.names[message[SENDER]] == client and message[SENDER] not in self.authentication_lst:
            self.database.delete_contact(message[SENDER], message[ACCOUNT_NAME])
            self.send(client, RESPONSE_200)
            return

        elif ACTION in message and message[ACTION] == USERS_REQUEST and SENDER in message and \
                self.names[message[SENDER]] == client and message[SENDER] not in self.authentication_lst:
            self.response = RESPONSE_202
            self.response[USERS_REQUEST] = [user[0] for user in self.database.users_list()]
            self.send(client, self.response)
            return

        elif ACTION in message and message[ACTION] == PUBLIC_KEY_REQUEST and ACCOUNT_NAME in message:
            response = RESPONSE_511
            response[DATA] = self.database.get_keys(message[ACCOUNT_NAME])
            if response[DATA]:
                try:
                    send_msg(client, response)
                except OSError:
                    self.get_name(client)
            else:
                response = RESPONSE_400
                response[ERROR] = 'Нет публичного ключа для данного пользователя'
                try:
                    send_msg(client, response)
                except OSError:
                    self.get_name(client)

        else:
            self.logger.error(f'created answer to client_ - {RESPONSE_400}')
            self.send(client, RESPONSE_400)
            return

    @Log()
    def send(self, *args):
        """
        send message function
        @param args: socket, message
        @return:
        """
        send_msg(*args)

    @Log()
    def get_name(self, client):
        """
        handler to detect and delete disconnected users
        @param client: username
        @return:
        """
        try:
            if client in self.connected_clients:
                self.connected_clients.remove(client)
            elif client in self.r_clients:
                self.r_clients.remove(client)
            elif client in self.w_clients:
                self.w_clients.remove(client)
            for i in self.names.items():
                if i[1] == client:
                    self.names.pop(i[0])
                    self.database.user_logout(i[0])
        except Exception as e:
            self.logger.warning(f'get_name exception {e}')


def main():
    """
    main function, create server_ GUI, get server_ settings
    @return:
    """
    server_config = configparser.ConfigParser()
    path = os.path.dirname(os.path.realpath(__file__))
    server_config.read(os.path.join(path, 'server.ini'))
    ip_addr, ip_port = arg_parser(server_config['SETTINGS']['DEFAULT_IP'], server_config['SETTINGS']['DEFAULT_PORT'])
    database = ServerDB(os.path.join(server_config['SETTINGS']['db_path'], server_config['SETTINGS']['db_file_name']))

    app = Server(ip_addr, ip_port, database)
    app.daemon = True
    app.start()
    gui_app = QApplication(sys.argv)
    main_window = MainGui()
    main_window.statusBar().showMessage('Server Working')
    main_window.active_users_table.setModel(gui_create_model(database))
    main_window.active_users_table.resizeColumnsToContents()
    main_window.active_users_table.resizeRowsToContents()

    def list_update():
        """
        func shows active users in GUI
        @return:
        """
        global new_connection
        if conflag_lock:
            main_window.active_users_table.setModel(
                gui_create_model(database))
            main_window.active_users_table.resizeColumnsToContents()
            main_window.active_users_table.resizeRowsToContents()
            with conflag_lock:
                new_connection = False

    def show_statistics():
        """
        func shows messages history in GUI
        @return:
        """
        global stat_window
        stat_window = HistoryWindow()
        stat_window.history_table.setModel(create_stat_model(database))
        stat_window.history_table.resizeColumnsToContents()
        stat_window.history_table.resizeRowsToContents()
        stat_window.show()

    def server_configuration():
        """
        func shows settings window in GUI
        @return:
        """
        global config_window
        config_window = ConfigWindow()
        config_window.db_path.insert(server_config['SETTINGS']['db_path'])
        config_window.db_file.insert(server_config['SETTINGS']['db_file_name'])
        config_window.port.insert(server_config['SETTINGS']['DEFAULT_PORT'])
        config_window.ip.insert(server_config['SETTINGS']['DEFAULT_IP'])
        config_window.save_btn.clicked.connect(save_server_config)

    def save_server_config():
        """
        check setting and save it in INI file
        @return:
        """
        global config_window
        message = QMessageBox()
        server_config['SETTINGS']['db_path'] = config_window.db_path.text()
        server_config['SETTINGS']['db_file_name'] = config_window.db_file.text()
        try:
            port = int(config_window.port.text())
        except ValueError:
            message.warning(config_window, 'Error', 'Port should be an integer')
        else:
            server_config['SETTINGS']['DEFAULT_IP'] = config_window.ip.text()
            if 1023 < port < 65536:
                server_config['SETTINGS']['DEFAULT_PORT'] = str(port)
                print(port)
                with open('server.ini', 'w') as conf:
                    server_config.write(conf)
                    message.information(
                        config_window, 'OK', 'Settings saved')
            else:
                message.warning(
                    config_window,
                    'Error',
                    'Port should be 1024 - 65536')

    timer = QTimer()
    timer.timeout.connect(list_update)
    timer.start(1000)

    main_window.refresh.triggered.connect(list_update)
    main_window.history.triggered.connect(show_statistics)
    main_window.server_config.triggered.connect(server_configuration)

    gui_app.exec_()


if __name__ == '__main__':
    main()



def check_ip(ip):
    """
    check if ip address is correct
    :param ip: str ip address
    :return: bool, true if ip is correct
    """
    for el in ip:
        if el.isdigit() or el == '.':
            pass
        else:
            return False
    return True


def check_first_octet(data):
    """
    check first octet of ip address
    :param data: int
    :return: bool
    """
    if 1 <= int(data) < 256:
        return True
    else:
        return False


def check_second_third_octets(data):
    """
    check second and third octets of ip address
    :param data: int
    :return: bool
    """
    if 0 <= int(data) < 256:
        return True
    else:
        return False


def check_forth_octet(data):
    """
    check forth octet of ip address
    :param data: int
    :return: bool
    """
    if 0 <= int(data) < 255:
        return True
    else:
        return False